# -- encoding: UTF-8 --

import sys

# From `contextlib` in Python 3.5.


class _RedirectStream:
    _stream = None

    def __init__(self, new_target):
        self._new_target = new_target
        # We use a list of old targets to make this CM re-entrant
        self._old_targets = []

    def __enter__(self):
        self._old_targets.append(getattr(sys, self._stream))
        setattr(sys, self._stream, self._new_target)
        return self._new_target

    def __exit__(self, exctype, excinst, exctb):
        setattr(sys, self._stream, self._old_targets.pop())


class redirect_stdout(_RedirectStream):
    _stream = "stdout"


class redirect_stderr(_RedirectStream):
    _stream = "stderr"



# -- encoding: UTF-8 --

import wrapt
from django.conf.urls import url
from django.contrib.auth.decorators import user_passes_test
from django.urls import reverse

from django_managerie.commands import get_commands
from django_managerie.views import ManagerieCommandView, ManagerieListView

superuser_required = user_passes_test(lambda u: u.is_active and u.is_superuser)


class Managerie:
    ignored_app_names = {
        'django.core',
        'django.contrib.staticfiles',
    }

    def __init__(self, admin_site):
        self.admin_site = admin_site

    def patch(self):
        if hasattr(self.admin_site, 'patched_by_managerie'):
            return
        self.admin_site.get_app_list = self.patched_get_app_list(self.admin_site.get_app_list)
        self.admin_site.get_urls = self.patched_get_urls(self.admin_site.get_urls)
        self.admin_site.patched_by_managerie = True

    def get_commands(self):
        return {
            app_config: commands
            for (app_config, commands)
            in get_commands().items()
            if app_config.name not in self.ignored_app_names
        }

    def get_commands_for_app_label(self, app_label):
        for app_config, commands in self.get_commands().items():
            if app_config.label == app_label:
                return commands
        return []

    @wrapt.decorator
    def patched_get_app_list(self, wrapped, instance, args, kwargs):
        request = args[0]
        app_list = wrapped(*args, **kwargs)
        if request.user.is_superuser:
            self._augment_app_list(app_list)
        return app_list

    @wrapt.decorator
    def patched_get_urls(self, wrapped, instance, args, kwargs):
        urls = wrapped(*args, **kwargs)
        return self._get_urls() + list(urls)

    def _augment_app_list(self, app_list):
        all_commands = {app_config.label: commands for (app_config, commands) in self.get_commands().items()}
        for app in app_list:
            commands = all_commands.get(app['app_label'], [])
            if commands:
                app.setdefault('models', []).append({
                    'perms': {'change': True},
                    'admin_url': reverse(
                        'admin:managerie_list',
                        kwargs={'app_label': app['app_label']},
                        current_app=self.admin_site.name,
                    ),
                    'name': 'Commands',
                    'object_name': '_ManagerieCommands_',
                })

    def _get_urls(self):
        return [
            url(
                '^managerie/(?P<app_label>.+?)/(?P<command>.+?)/$',
                superuser_required(ManagerieCommandView.as_view(managerie=self)),
                name='managerie_command',
            ),
            url(
                '^managerie/(?P<app_label>.+?)/$',
                superuser_required(ManagerieListView.as_view(managerie=self)),
                name='managerie_list',
            ),
            url(
                '^managerie/$',
                superuser_required(ManagerieListView.as_view(managerie=self)),
                name='managerie_list_all',
            ),
        ]

    @property
    def urls(self):
        return (self._get_urls(), 'admin', self.admin_site.name)



# -- encoding: UTF-8 --
from argparse import _HelpAction, _StoreAction, _StoreTrueAction

from django import forms

FIELD_CLASS_MAP = {'int': forms.IntegerField, }


class ArgumentParserForm(forms.Form):

    IGNORED_DESTS = {
        'pythonpath',
        'settings',
        'no_color',
        'traceback',
    }

    def __init__(self, *, parser, **kwargs):
        """
        :type parser: argparse.ArgumentParser
        """
        self.parser = parser
        super(ArgumentParserForm, self).__init__(**kwargs)
        for action in self.parser._actions:
            print(action)
            if isinstance(action, _HelpAction):
                continue
            if action.dest in self.IGNORED_DESTS:
                continue
            field_cls = None
            field_kwargs = dict(
                initial=action.default,
                label=action.dest.replace('_', ' ').title(),
                help_text=action.help,
                required=action.required,
            )
            if isinstance(action, _StoreTrueAction):
                field_cls = forms.BooleanField
            elif isinstance(action, _StoreAction):
                field_cls = FIELD_CLASS_MAP.get(action.type, forms.Field)
            if field_cls:
                self.fields[action.dest] = field_cls(**field_kwargs)

            # TODO: Probably support for more fields :)



import io
import traceback

import time
from itertools import chain

from django.apps import apps
from django.views.generic import TemplateView, FormView

from django_managerie.forms import ArgumentParserForm
from .compat import redirect_stdout, redirect_stderr


class MenagerieBaseMixin:
    managerie = None

    def get_app(self):
        if hasattr(self, '_app'):
            return self._app
        if 'app_label' in self.kwargs:
            self._app = apps.get_app_config(self.kwargs['app_label'])
            return self._app


class ManagerieListView(MenagerieBaseMixin, TemplateView):
    template_name = 'django_managerie/admin/list.html'

    def get_context_data(self, **kwargs):
        context = super(ManagerieListView, self).get_context_data(**kwargs)
        context['app'] = app = self.get_app()
        context['title'] = '%s \u2013 Commands' % (app.verbose_name if app else 'All Apps')
        context['commands'] = sorted(
            (
                self.managerie.get_commands_for_app_label(app.label).values()
                if app
                else chain(*(app_commands.values() for app_commands in self.managerie.get_commands().values()))
            ),
            key=lambda cmd: cmd.full_title
        )
        return context


class ManagerieCommandView(MenagerieBaseMixin, FormView):
    template_name = 'django_managerie/admin/command.html'

    def get_command_object(self):
        return self.managerie.get_commands_for_app_label(self.get_app().label)[self.kwargs['command']]

    def get_form(self, form_class=None):
        cmd = self.get_command_object().get_command_instance()
        parser = cmd.create_parser('django', self.kwargs['command'])
        return ArgumentParserForm(parser=parser, **self.get_form_kwargs())

    def get_context_data(self, **kwargs):
        context = super(ManagerieCommandView, self).get_context_data(**kwargs)
        context['app'] = self.get_app()
        context['command'] = cmd = self.get_command_object()
        context['command_help'] = cmd.get_command_instance().help
        context['title'] = cmd.full_title
        return context

    def form_valid(self, form):
        # This mimics BaseCommand.run_from_argv():
        options = dict(form.cleaned_data)
        args = options.pop('args', ())  # "Move positional args out of options to mimic legacy optparse"
        stdout = io.StringIO()
        stderr = io.StringIO()
        error = None
        error_tb = None
        t0 = time.time()
        co = self.get_command_object()
        with redirect_stdout(stdout), redirect_stderr(stderr):
            options.update({
                'traceback': True,
                'no_color': True,
                'stdout': stdout,
                'stderr': stderr,
            })
            cmd = co.get_command_instance()
            try:
                cmd.execute(*args, **options)
            except SystemExit as se:  # We don't want any stray sys.exit()s to quit the app server
                stderr.write('<exit: %s>' % se)
            except Exception as exc:
                error = exc
                error_tb = traceback.format_exc()
        context = self.get_context_data(
            executed=True,
            form=form,
            error=error,
            error_tb=error_tb,
            stdout=stdout.getvalue(),
            stderr=stderr.getvalue(),
            duration=(time.time() - t0),
        )
        return self.render_to_response(context=context, status=(400 if error else 200))



#!/usr/bin/env python

from setuptools import setup, find_packages

setup(
    name='django-managerie',
    version='0.0',
    author='Aarni Koskela',
    author_email='akx@iki.fi',
    packages=find_packages(include=('django_managerie', 'django_managerie*')),
    include_package_data=True,
    install_requires=['Django', 'wrapt'],
)



# -- encoding: UTF-8 --
import os
from collections import defaultdict, OrderedDict

from django.apps import apps
from django.core.management import load_command_class, find_commands
from django.urls import reverse
from django.utils import lru_cache


class ManagementCommand:
    def __init__(self, app_config, name):
        self.app_config = app_config
        self.name = name
        self.title = self.name.replace('_', ' ').title()

    @property
    def url(self):
        return reverse(
            'admin:managerie_command',
            kwargs={'app_label': self.app_config.label, 'command': self.name},
        )

    def get_command_instance(self):
        return load_command_class(self.app_config.label, self.name)

    @property
    def full_title(self):
        return '%s \u2013 %s' % (self.app_config.verbose_name, self.title)


@lru_cache.lru_cache(maxsize=None)
def get_commands():
    # Logic filched from django.core.management.get_commands(), but expressed in a saner way.
    apps_to_commands = defaultdict(OrderedDict)

    for app_config in apps.get_app_configs():
        path = os.path.join(app_config.path, 'management')
        for command_name in find_commands(path):
            apps_to_commands[app_config][command_name] = ManagementCommand(
                app_config=app_config,
                name=command_name,
            )
    return apps_to_commands
